from ohm2_pushes_light import settings as ohm2_pushes_light_settings

ONESIGNAL_ENABLED = getattr(ohm2_pushes_light_settings, 'ONESIGNAL_ENABLED')
FIREBASE_ENABLED = getattr(ohm2_pushes_light_settings, 'FIREBASE_ENABLED')


from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.translation import ugettext as _
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.definitions import RunException
from ohm2_pushes_light.gateways.onesignal import utils as onesignal_utils
from ohm2_pushes_light.gateways.firebase import utils as firebase_utils
from . import models as ohm2_pushes_light_models
from . import errors as ohm2_pushes_light_errors
from . import settings
from . import definitions as ohm2_pushes_light_definitions
import os, time, random


random_string = "RL6G7j5snwNojousZp25qqKZu38NMi3w"


def create_message(user, title, body, **kwargs):
	kwargs["user"] = user
	kwargs["title"] = title.strip()
	kwargs["body"] = body.strip()
	return h_utils.db_create(ohm2_pushes_light_models.Message, **kwargs)

def get_message(**kwargs):
	return h_utils.db_get(ohm2_pushes_light_models.Message, **kwargs)

def get_or_none_message(**kwargs):
	return h_utils.db_get_or_none(ohm2_pushes_light_models.Message, **kwargs)

def filter_message(**kwargs):
	return h_utils.db_filter(ohm2_pushes_light_models.Message, **kwargs)

def q_message(q):
	return h_utils.db_q(ohm2_pushes_light_models.Message, q)

def create_dummy_message(user, title, body, **kwargs):
	message = ohm2_pushes_light_definitions.DummyMessage(user, title, body, **kwargs)
	return message

def get_user_devices(user, **options):
	providers = (
		{
			"gateway": settings.GATEWAY_ONESIGNAL[0],
			"getter": onesignal_utils.filter_device,
		},
	)
	querysets = {}
	for p in providers:
		querysets[p["gateway"]] = p["getter"](user = user).order_by("-last_update")
	return querysets


def send_notification(message, **options):
	summary = {}
	for gateway, queryset in get_user_devices(message.user).items():

		if gateway == settings.GATEWAY_ONESIGNAL[0]:			
			player_ids = [device.player_id for device in queryset]
			if len(player_ids) == 0:
				continue			
			summary[gateway] = onesignal_utils.send_notifications(message, player_ids, **options.get("onesignal_options", {}))

		elif gateway == settings.GATEWAY_FIREBASE[0]:
			pass

	return summary		

def mark_message_as_read(message, **options):
	return h_utils.db_update(message, read = True)

"""
def create_model(**kwargs):
	return h_utils.db_create(ohm2_pushes_light_models.Model, **kwargs)

def get_model(**kwargs):
	return h_utils.db_get(ohm2_pushes_light_models.Model, **kwargs)

def get_or_none_model(**kwargs):
	return h_utils.db_get_or_none(ohm2_pushes_light_models.Model, **kwargs)

def filter_model(**kwargs):
	return h_utils.db_filter(ohm2_pushes_light_models.Model, **kwargs)

def q_model(q):
	return h_utils.db_q(ohm2_pushes_light_models.Model, q)			
"""



from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.translation import ugettext as _
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.definitions import RunException
from . import models as onesignal_models
from . import errors as onesignal_errors
from . import settings
import simplejson as json
import os, time, random, requests


random_string = "L4grZ9Lzlr7eByGMzCsullCuUkk2b8BO"

def parse_device_attributes(**kwargs):
	attributes = {}
	
	user = kwargs.get("user", None)
	if user is not None:
		attributes["user"] = user
	

	platform = kwargs.get("platform", None)
	if platform is not None:
		attributes["platform"] = int(platform)

		
	player_id = kwargs.get("player_id", None)
	if player_id is not None:
		attributes["player_id"] = player_id.strip()
	

	push_token = kwargs.get("push_token", None)
	if push_token is not None:
		attributes["push_token"] = push_token.strip()

	return attributes


def create_device(user, platform, player_id, push_token, **kwargs):
	kwargs["user"] = user
	
	kwargs["gateway"] = onesignal_models.Device.gateway_onesignal[0]
	
	kwargs["platform"] = platform
	kwargs["player_id"] = player_id
	kwargs["push_token"] = push_token
	for key, value in parse_device_attributes(**kwargs).items():
		kwargs[key] = value
	return h_utils.db_create(onesignal_models.Device, **kwargs)

def get_device(**kwargs):
	return h_utils.db_get(onesignal_models.Device, **kwargs)

def get_or_none_device(**kwargs):
	return h_utils.db_get_or_none(onesignal_models.Device, **kwargs)

def filter_device(**kwargs):
	return h_utils.db_filter(onesignal_models.Device, **kwargs)

def q_device(q):
	return h_utils.db_q(onesignal_models.Device, q)	

def update_device(entry, **kwargs):
	attributes = {}
	for key, value in parse_device_attributes(**kwargs).items():
		attributes[key] = value
	return h_utils.db_update(entry, **attributes)

def get_platform_ios():
	return onesignal_models.Device.platform_ios

def get_platform_android():
	return onesignal_models.Device.platform_android

def send_notifications(message, player_ids, **options):
	contents = options.get("contents", {
		"en": message.title,
	})
	payload = {
		"app_id": settings.APP_ID,
		"include_player_ids": player_ids,
		"contents": contents,
		"data": message.get_onesignal_data(),
	}
	res = requests.post(settings.API_NOTIFICATIONS_URL, headers = settings.HEADERS, data = json.dumps(payload))
	if res.status_code != 200:
		raise RunException(**onesignal_errors.INVALID_NOTIFICATION_STATUS_CODE)
	ret = res.json()

	pushed = True
	return pushed


"""
def create_model(**kwargs):
	return h_utils.db_create(onesignal_models.Model, **kwargs)

def get_model(**kwargs):
	return h_utils.db_get(onesignal_models.Model, **kwargs)

def get_or_none_model(**kwargs):
	return h_utils.db_get_or_none(onesignal_models.Model, **kwargs)

def filter_model(**kwargs):
	return h_utils.db_filter(onesignal_models.Model, **kwargs)

def q_model(q):
	return h_utils.db_q(onesignal_models.Model, q)			
"""



from django.db import models
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from django.utils.translation import ugettext as _
from ohm2_handlers_light.models import BaseModel
from ohm2_pushes_light import models as ohm2_pushes_light_models
from . import managers
from . import settings



"""
class Device(ohm2_pushes_light_models.Device):
	user = models.ForeignKey(User, on_delete = models.CASCADE, related_name = "firebase_device_user")
	token = models.CharField(max_length = settings.STRING_DOUBLE)

	def __str__(self):
		return "[%s][%s][Token: %s]" % (self.user.get_username(), self.platform_str, self.token)
"""


from rest_framework import serializers
from . import models as onesignal_models
from . import settings


"""
class Serializer(serializers.ModelSerializer):
	class Meta:
		model = onesignal_models.Model
		fields = (
			'identity',
			'created',
			'last_update',
		)
"""		


from django.shortcuts import render, redirect
from django.urls import reverse
from django.http import HttpResponse, JsonResponse
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from rest_framework.authentication import SessionAuthentication, TokenAuthentication
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.decorators import authentication_classes, permission_classes, api_view
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.parsers import get_as_or_get_default
from ohm2_pushes_light import models as ohm2_pushes_light_models
from . import dispatcher
from . import settings



def process_gateways_onesignal_register_device(request, platform):
	keys = (
		("player_id", "player_id", ""),
		("push_token", "push_token", ""),
		("platform", h_utils.random_string(10), platform),
	)
	res, error = dispatcher.gateways_onesignal_register_device(request, get_as_or_get_default(request.data, keys))
	if error:
		return JsonResponse({"error": error.regroup()})
	return JsonResponse(res)


@api_view(['POST'])
@authentication_classes((TokenAuthentication, SessionAuthentication))
@permission_classes((IsAuthenticated,))
def gateways_onesignal_register_device_android(request):
	"""
	Register Device (Android)

	
	__Inputs__:

		- player_id (string, required): device's player id
		- push_token (string, required): device's push token

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if the signup completed succesfully.

	__Notes__:

		- 
	
	"""
	return process_gateways_onesignal_register_device(request, ohm2_pushes_light_models.Device.platform_android[0])
	

@api_view(['POST'])
@authentication_classes((TokenAuthentication, SessionAuthentication))
@permission_classes((IsAuthenticated,))
def gateways_onesignal_register_device_ios(request):
	"""
	Register Device (iOS)

	
	__Inputs__:

		- player_id (string, required): device's player id
		- push_token (string, required): device's push token

	__Output__:

		- error (json-dict): describes the error (if known) with a code (integer) and a message (string).
		- ret (boolean): true if the signup completed succesfully.

	__Notes__:

		- 
	
	"""
	return process_gateways_onesignal_register_device(request, ohm2_pushes_light_models.Device.platform_ios[0])


	
	






# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-10 17:15
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Message',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identity', models.CharField(max_length=2048, unique=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('last_update', models.DateTimeField(default=django.utils.timezone.now)),
                ('title', models.CharField(max_length=512)),
                ('body', models.TextField()),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default={}, null=True)),
                ('read', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.conf.urls import url, include
from . import settings
from . import views

app_name = "ohm2_pushes_light_api_v1"

urlpatterns = []

if settings.ONESIGNAL_ENABLED:
	urlpatterns += [
		url(r'^gateways/onesignal/register-device/android/$', views.gateways_onesignal_register_device_android, name = 'gateways_onesignal_register_device_android'),
		url(r'^gateways/onesignal/register-device/ios/$', views.gateways_onesignal_register_device_ios, name = 'gateways_onesignal_register_device_ios'),
	]




from django.core.management.base import BaseCommand, CommandError
from ohm2_handlers_light import utils as h_utils
from firebase import utils as firebase_utils
import os

class Command(BaseCommand):
	
	def add_arguments(self, parser):
		pass #parser.add_argument('-f', '--foo')

	def handle(self, *args, **options):
		# foo = options["foo"]
		pass



from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _
from django.contrib import messages
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.definitions import RunException
from . import settings
from . import utils as onesignal_utils
from .decorators import onesignal_safe_request
import os, time, random


@onesignal_safe_request
def index(request, params):
	p = h_utils.cleaned(params, (
					 	))
	
	
	ret = {
	}
	return ret


from django.contrib import admin
from . import models as onesignal_models


@admin.register(onesignal_models.Device)
class Device(admin.ModelAdmin):
	
	exclude = (
		'identity',
		'last_update',
		'code',
	)

	

	def save_model(self, request, obj, form, change):
		obj = form.instance
		if not change or len(obj.identity) == 0:
			obj.identity = h_utils.db_unique_random(type(obj))
		
		super(type(self), self).save_model(request, obj, form, change)


from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.translation import ugettext as _
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.definitions import RunException
from . import models as firebase_models
from . import errors as firebase_errors
from . import settings
import os, time, random


random_string = "v91XnXLG4n4eYSugPuwd7xYTVknj43rH"



"""
def create_model(**kwargs):
	return h_utils.db_create(firebase_models.Model, **kwargs)

def get_model(**kwargs):
	return h_utils.db_get(firebase_models.Model, **kwargs)

def get_or_none_model(**kwargs):
	return h_utils.db_get_or_none(firebase_models.Model, **kwargs)

def filter_model(**kwargs):
	return h_utils.db_filter(firebase_models.Model, **kwargs)

def q_model(q):
	return h_utils.db_q(firebase_models.Model, q)			
"""



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-10 17:15
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identity', models.CharField(max_length=2048, unique=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('last_update', models.DateTimeField(default=django.utils.timezone.now)),
                ('platform', models.IntegerField(choices=[(1, 'iOS'), (2, 'Android'), (3, 'Browser')])),
                ('gateway', models.IntegerField(choices=[(1, 'OneSignal'), (2, 'Firebase')], default=1)),
                ('player_id', models.CharField(max_length=2048)),
                ('push_token', models.CharField(max_length=2048)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
from django.contrib.postgres.fields import JSONField
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from django.utils.translation import ugettext as _
from ohm2_handlers_light.models import BaseModel
from . import managers
from . import settings




class Message(BaseModel):
	user = models.ForeignKey(User, on_delete = models.CASCADE)
	title = models.CharField(max_length = settings.STRING_NORMAL)
	body = models.TextField()	
	data = JSONField(null = True, blank = True, default = {})
	read = models.BooleanField(default = False)
	
	def __str__(self):
		return "%s -> [%s] at [%s]" % (self.title, self.user.get_username(), self.created)

	def get_onesignal_data(self):
		return {
			"body": self.body,
			"data": self.data,
		}

	
				
	


class Device(BaseModel):
	platform_ios = (1, "iOS")
	platform_android = (2, "Android")
	platform_browser = (3, "Browser")
	
	platforms = (
		platform_ios,
		platform_android,
		platform_browser,
	)
	platform = models.IntegerField(choices = platforms)

	gateway_onesignal = settings.GATEWAY_ONESIGNAL
	gateway_firebase = settings.GATEWAY_FIREBASE
	gateways = settings.GATEWAYS
	gateway = models.IntegerField(choices = gateways, default = settings.DEFAULT_GATEWAY[0])
	
	class Meta:
		abstract = True

	@property
	def is_ios(self):
		return self.platform == self.platform_ios[0]

	@property
	def is_android(self):
		return self.platform == self.platform_android[0]

	@property
	def is_browser(self):
		return self.platform == self.platform_browser[0]		

	@property
	def is_onesignal(self):
		return self.gateway == self.gateway_onesignal[0]

	@property
	def platform_str(self):
		for platform in self.platforms:
			if platform[0] == self.platform:
				return platform[1]
		return ""


from django.shortcuts import render, redirect
from django.core.urlresolvers import reverse
from django.http import HttpResponse, JsonResponse
from django.contrib.auth.decorators import login_required
from ohm2_handlers_light.parsers import get_as_or_get_default
from . import dispatcher


def index(request):
	keys = (
	)
	ret, error = dispatcher.index(request, get_as_or_get_default(request.GET, keys))
	if error:
		return HttpResponse("error")	
	return render(request, "onesignal/index.html", {"ret" : ret})



from . import settings


class DummyMessage(object):

	def __init__(self, user, title, body, **options):
		self.user = user
		self.title = title
		self.body = body
		self.data = options.get("data", {})
		self.read = options.get("read", False)

	def get_onesignal_data(self):
		return {
			"body": self.body,
			"data": self.data,
		}	


from ohm2_handlers_light import utils as h_utils
from . import settings
import os


"""
def upload_to(instance, filename):	
	filename_full = h_utils.random_string(15) + "." + filename.rsplit(".", 1)[-1]
	return os.path.join(settings.UPLOAD_TO, filename_full)


def post_delete(sender, **kwargs):	
	try:
		instance = kwargs['instance']
		os.remove( os.path.join(settings.MEDIA_ROOT, instance.avatar.name) )
	except Exception as e:
		pass
"""


from ohm2_handlers_light import utils as h_utils
from ohm2_pushes_light.decorators import ohm2_pushes_light_safe_request
from ohm2_pushes_light import utils as ohm2_pushes_light_utils
from ohm2_pushes_light import settings
from . import errors as api_v1_errors
from . import settings as api_v1_settings



@ohm2_pushes_light_safe_request
def gateways_onesignal_register_device(request, params, **pipeline_options):
	p = h_utils.cleaned(params, (
							("string", "player_id", 1),
							("string", "push_token", 1),
							("integer", "platform", None),
						))

	platform, player_id, push_token = p["platform"], p["player_id"], p["push_token"]

	device = ohm2_pushes_light_utils.onesignal_utils.filter_device(player_id = player_id).first()
	if device:
		device = ohm2_pushes_light_utils.onesignal_utils.update_device(
			device,
			user = request.user,
			platform = platform,
			push_token = push_token,
		)
	else:
		device = ohm2_pushes_light_utils.onesignal_utils.create_device(request.user, platform, player_id, push_token)
	
	
	res = {
		"error" : None,
		"ret" : True,
	}
	return res







from django.core.urlresolvers import reverse
from django.shortcuts import render, redirect
from django.contrib.auth.models import User
try:
	from django.utils.deprecation import MiddlewareMixin
except ImportError:
	MiddlewareMixin = object
from . import utils as ohm2_pushes_light_utils



class Ohm2_Pushes_LightMiddleware(MiddlewareMixin):
	
	def process_request(self, request):
		
		if request.user.is_authenticated():

			is_staff = request.user.is_staff
			is_superuser = request.user.is_superuser
			
			if is_superuser or is_staff:
				return None

				
		return None
	
	
	def process_response(self, request, response):
		
		return response
	



# Generated by Django 2.0.1 on 2018-11-19 20:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('onesignal', '0002_auto_20180928_1418'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='identity',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]
